package com.emato.cus.supervise.biz.acqShelfLoad.down;

import com.emato.cus.supervise.biz.CusWmsTask;
import com.emato.cus.supervise.biz.OmsDataImpl;
import com.emato.cus.supervise.constant.EmtRecordTimeEnum;
import com.emato.cus.supervise.domain.WmsAcqShelfLoad;
import com.emato.cus.supervise.domainCus.CusAcqShelfLoad04;
import com.emato.cus.supervise.domainOms.OmsMerchInfo;
import com.emato.cus.supervise.domainOms.OmsProductInfo;
import com.emato.cus.supervise.domainOmsShop.OmsShopAcqShelfLoad;
import com.emato.cus.supervise.util.GsonUtil;
import com.emato.cus.supervise.util.LocalDateTimeUtils;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * 下架
 * 特别情况，门店出场返区，必须删除在CUS中间表中已经存在的库位（skuNo和seatNo联合唯一）数据
 * @author zengjunlin
 * @version 1.0
 * 2018-02-03 11:25
 */
@Component
public class AcqShelfLoadDownBiz implements CusWmsTask{

    private static final Logger logger = LoggerFactory.getLogger(AcqShelfLoadDownBiz.class);

    @Autowired
    private AcqShelfLoadDownRecordTime acqShelfLoadDownRecordTime;

    @Autowired
    private AcqShelfLoadDownCusData acqShelfLoadDownCusData;

    @Autowired
    private AcqShelfLoadDownWmsData acqShelfLoadDownWmsData;

    @Autowired
    private AcqShelfLoadDownDataConversion acqShelfLoadDownDataConversion;

    @Autowired
    private OmsDataImpl omsDataImpl;


    @Autowired
    private AcqShelfLoadOutDownOmsShopData acqShelfLoadOutDownOmsShopData;
    @Autowired
    private AcqShelfLoadOutDownOmsShopDataConversion acqShelfLoadOutDownOmsShopDataConversion;
    @Autowired
    private AcqShelfLoadSaleDownOmsShopData acqShelfLoadSaleDownOmsShopData;
    @Autowired
    private AcqShelfLoadSaleDownOmsShopDataConversion acqShelfLoadSaleDownOmsShopDataConversion;

    @Autowired
    private AcqShelfLoadDownOmsShopCusData acqShelfLoadDownOmsShopCusData;


    private static final String DATA_TYPE_INFO = "[" + EmtRecordTimeEnum.ACQ_SHELF_LOAD_DOWN.getCode() + "]" + EmtRecordTimeEnum.ACQ_SHELF_LOAD_DOWN.getName()
            + "，[" + EmtRecordTimeEnum.OMS_SHOP_ACQ_SHELF_LOAD_DOWN.getCode() + "]" + EmtRecordTimeEnum.OMS_SHOP_ACQ_SHELF_LOAD_DOWN.getName();

    private static final String WMS_DATA_TYPE_INFO = "[" + EmtRecordTimeEnum.ACQ_SHELF_LOAD_DOWN.getCode() + "]" + EmtRecordTimeEnum.ACQ_SHELF_LOAD_DOWN.getName();
    private static final String SHOP_DATA_TYPE_INFO = "[" + EmtRecordTimeEnum.OMS_SHOP_ACQ_SHELF_LOAD_DOWN.getCode() + "]" + EmtRecordTimeEnum.OMS_SHOP_ACQ_SHELF_LOAD_DOWN.getName();
    private static final String SHOP_DATA_TYPE_INFO_SALE_DOWN = "[" + EmtRecordTimeEnum.OMS_SHOP_ACQ_SHELF_LOAD_SALE_DOWN.getCode() + "]" + EmtRecordTimeEnum.OMS_SHOP_ACQ_SHELF_LOAD_SALE_DOWN.getName();



    /**
     * 任务入口
     */
    @Override
    public void task() {
        // 获取扫描时间
        String oldThisTime = acqShelfLoadDownRecordTime.getEmtRecordTime();
        if (StringUtils.isBlank(oldThisTime)) {
            logger.info("--- 【" + DATA_TYPE_INFO + "】未获取到本次扫描时间, 本次扫描提前结束！！！");
            //下次扫描时间
            acqShelfLoadDownRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                    LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN));
            return;
        }


        // 扫描WMS 数据
        List<WmsAcqShelfLoad> wmsList = acqShelfLoadDownWmsData.getWmsData(oldThisTime);
        // 执行wms相关操作
        boolean wmsOp = true;
        if (wmsList == null || wmsList.size() == 0) {
            logger.info("--- 【" + DATA_TYPE_INFO + "】获取到本次扫描数据为空, 不执行写数据操作, 本次扫描提前结束！！！");
            // 不执行WMS相关操作
            wmsOp = false;
            /*acqShelfLoadDownRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                    oldThisTime);
            return;*/
        }

        // req_20200430001 门店库存海关监管
        // 出场 下架
        List<OmsShopAcqShelfLoad> omsShopList = acqShelfLoadOutDownOmsShopData.getOmsShopData(oldThisTime);
        boolean omsShopOp = true;
        if (omsShopList == null || omsShopList.size() == 0) {
            logger.info("--- 【" + SHOP_DATA_TYPE_INFO + "】获取到本次扫描数据为空, 不执行写数据操作.");
            omsShopOp = false;
        }

        // 销售 下架
        List<OmsShopAcqShelfLoad> omsShopSaleList = acqShelfLoadSaleDownOmsShopData.getOmsShopData(oldThisTime);
        boolean omsShopSaleOp = true;
        if (omsShopList == null || omsShopList.size() == 0) {
            logger.info("--- 【" + SHOP_DATA_TYPE_INFO_SALE_DOWN + "】获取到本次扫描数据为空, 不执行写数据操作.");
            omsShopSaleOp = false;
        }

        if (!wmsOp && !omsShopOp && !omsShopSaleOp) {
            logger.info("--- 【" + DATA_TYPE_INFO + "】获取到本次扫描数据为空, 不执行写数据操作, 提前结束");
            //下次扫描时间
            acqShelfLoadDownRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                    oldThisTime);
            return;
        }

        if (wmsOp) {
            operateWms(wmsList, oldThisTime);
        }

        if (omsShopOp) {
            operateOmsShop(omsShopList, oldThisTime);
        }

        if (omsShopSaleOp) {
            operateOmsShopSale(omsShopSaleList, oldThisTime);
        }

        // 更新扫描时间
        boolean bl = acqShelfLoadDownRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                oldThisTime);
    }


    /**
     * 写入WMS数据
     * @param wmsList
     * @param oldThisTime
     */
    private void operateWms(List<WmsAcqShelfLoad> wmsList, String oldThisTime) {
        List<String> merchSnList = Lists.newArrayList();
        wmsList.forEach(i -> merchSnList.add(i.getMerchSn()));

        // 查询oms系统数据。
        List<OmsMerchInfo> omsList = omsDataImpl.getOmsMerchInfoData(merchSnList);

        if (omsList == null || omsList.isEmpty()) {
            logger.error("oms扫描到的数据" + GsonUtil.toJson(wmsList));
            logger.info("--- 【" + WMS_DATA_TYPE_INFO + "】获取到本次扫描OMS数据为空, 不执行写数据操作, 本次扫描提前结束！！！");
            //下次扫描时间
            acqShelfLoadDownRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                    oldThisTime);
            return;
        }


        List<String> skuList = Lists.newArrayList();
        wmsList.forEach(i -> skuList.add(i.getSkuNo()));

        List<OmsProductInfo> productList = omsDataImpl.getOmsProductData(skuList);

        if (productList == null || productList.isEmpty()) {
            logger.error("oms扫描到的产品数据" + GsonUtil.toJson(productList));
            logger.info("--- 【" + DATA_TYPE_INFO + "】获取到OMS产品数据为空, 不执行写数据操作, 本次扫描提前结束！！！");
            //下次扫描时间
            acqShelfLoadDownRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                    oldThisTime);
            return;
        }

        Map params = Maps.newHashMap();
        params.put("wmsList",wmsList);
        params.put("omsList", omsList);
        params.put("productInfoList", productList);
        //组装CUS 数据
        List<CusAcqShelfLoad04> cusList = acqShelfLoadDownDataConversion.conversion(params);

        // 写CUS 数据
        int res = acqShelfLoadDownCusData.writeCus(cusList);
    }


    /**
     * 写入OMS shop
     * 出场
     * @param omsShopList
     * @param oldThisTim
     */
    private void operateOmsShop(List<OmsShopAcqShelfLoad> omsShopList, String oldThisTim) {
        Map params = Maps.newHashMap();
        params.put("omsShopList",omsShopList);

        //将oms shop数据转换cus
        List<CusAcqShelfLoad04> cusList = acqShelfLoadOutDownOmsShopDataConversion.conversion(params);

        // 写CUS 数据
        int res = acqShelfLoadDownCusData.writeCus(cusList);

        // 门店返区出场，必须要删除门店库位数据
        int delRes = acqShelfLoadDownOmsShopCusData.deteleteCus(cusList);
    }

    /**
     * 写入OMS shop
     * 销售
     * @param omsShopList
     * @param oldThisTim
     */
    private void operateOmsShopSale(List<OmsShopAcqShelfLoad> omsShopList, String oldThisTim) {
        Map params = Maps.newHashMap();
        params.put("omsShopList",omsShopList);

        //将oms shop数据转换cus
        List<CusAcqShelfLoad04> cusList = acqShelfLoadSaleDownOmsShopDataConversion.conversion(params);

        // 写CUS 数据
        int res = acqShelfLoadDownCusData.writeCus(cusList);
    }

}
