package com.emato.cus.supervise.config.ds.second;

import com.emato.cus.supervise.config.ds.primary.PrimaryMapperConfig;
import com.emato.cus.supervise.util.YmlUtils;
import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableList;
import org.mybatis.spring.mapper.MapperScannerConfigurer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.transaction.annotation.EnableTransactionManagement;


/**
 * Mybatis Mapper接口扫描配置
 *
 * @author Scott Chen
 * @date 2017/4/23
 */
@Configuration
public class SecondMapperConfig {
    private static final Logger logger = LoggerFactory.getLogger(SecondMapperConfig.class);

    @Bean
    public MapperScannerConfigurer secondMapperScannerConfigurer() {
        MapperScannerConfigurer mapperScannerConfigurer = new MapperScannerConfigurer();
        //Mapper接口包
        mapperScannerConfigurer.setBasePackage(mapperPackageByYml());
        //要使用的数据源
        mapperScannerConfigurer.setSqlSessionTemplateBeanName("secondSqlSessionTemplate");
        return mapperScannerConfigurer;
    }

    /**
     * 添加要扫描的Mapper接口包
     * 以逗号分隔的字符串
     *
     * @return
     */
    private String mapperPackageByYml() {
        String path = SecondMybatisConfigProperties.MYBATIS_MAPPER_CONFIG_YML_PATH;
        logger.debug("yml: {}", path);

        String key = "mybatis.second.mapper-scanner-package";
        return YmlUtils.instance().getValue(key, path);
    }

}
